from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import CoreML


class TestMLModelConfiguration(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreML.MLComputeUnits)

    def test_constants(self):
        self.assertEqual(CoreML.MLComputeUnitsCPUOnly, 0)
        self.assertEqual(CoreML.MLComputeUnitsCPUAndGPU, 1)
        self.assertEqual(CoreML.MLComputeUnitsAll, 2)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertResultIsBOOL(
            CoreML.MLModelConfiguration.allowLowPrecisionAccumulationOnGPU
        )
        self.assertArgIsBOOL(
            CoreML.MLModelConfiguration.setAllowLowPrecisionAccumulationOnGPU_, 0
        )

    @expectedFailure
    def test_methods10_15_missing(self):
        self.assertResultIsBOOL(CoreML.MLModelDescription.isUpdatable)



from PyObjCTools.TestSupport import TestCase
import CoreML


class TestMLImageSizeConstraintType(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreML.MLImageSizeConstraintType)

    def test_constants(self):
        self.assertEqual(CoreML.MLImageSizeConstraintTypeUnspecified, 0)
        self.assertEqual(CoreML.MLImageSizeConstraintTypeEnumerated, 2)
        self.assertEqual(CoreML.MLImageSizeConstraintTypeRange, 3)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLFeatureValue_MLImageConversion(TestCase):
    def test_typed_enum(self):
        self.assertIsTypedEnum(CoreML.MLFeatureValueImageOption, str)

    @min_os_level("10.15")
    def test_constants10_15(self):
        self.assertIsInstance(CoreML.MLFeatureValueImageOptionCropRect, str)
        self.assertIsInstance(CoreML.MLFeatureValueImageOptionCropAndScale, str)

    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithImageAtURL_pixelsWide_pixelsHigh_pixelFormatType_options_error_,
            5,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithImageAtURL_constraint_options_error_,
            3,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithCGImage_pixelsWide_pixelsHigh_pixelFormatType_options_error_,
            5,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithCGImage_constraint_options_error_, 3
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithImageAtURL_orientation_pixelsWide_pixelsHigh_pixelFormatType_options_error_,
            6,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithImageAtURL_orientation_constraint_options_error_,
            4,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithCGImage_orientation_pixelsWide_pixelsHigh_pixelFormatType_options_error_,
            6,
        )
        self.assertArgIsOut(
            CoreML.MLFeatureValue.featureValueWithCGImage_orientation_constraint_options_error_,
            4,
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLModel_MLModelCompilation(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(CoreML.MLModel.compileModelAtURL_error_, 1)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLFeatureDescription(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(CoreML.MLFeatureDescription.isOptional)
        # self.assertArgIsBOOL(CoreML.MLFeatureDescription.setOptional_, 0)
        self.assertResultIsBOOL(CoreML.MLFeatureDescription.isAllowedValue_)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLFeatureDescription(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreML.MLModelError)

    def testConstants(self):
        self.assertEqual(CoreML.MLModelErrorGeneric, 0)
        self.assertEqual(CoreML.MLModelErrorFeatureType, 1)
        self.assertEqual(CoreML.MLModelErrorIO, 3)
        self.assertEqual(CoreML.MLModelErrorCustomLayer, 4)
        self.assertEqual(CoreML.MLModelErrorCustomModel, 5)
        self.assertEqual(CoreML.MLModelErrorUpdate, 6)
        self.assertEqual(CoreML.MLModelErrorParameters, 7)
        self.assertEqual(CoreML.MLModelErrorModelDecryptionKeyFetch, 8)
        self.assertEqual(CoreML.MLModelErrorModelDecryption, 9)
        self.assertEqual(CoreML.MLModelErrorModelCollection, 10)

    @min_os_level("10.13")
    def testConstants10_13(self):
        self.assertIsInstance(CoreML.MLModelErrorDomain, str)



from PyObjCTools.TestSupport import TestCase
import CoreML


class TestMLUpdateProgressEvent(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreML.MLUpdateProgressEvent)

    def test_constants(self):
        self.assertEqual(CoreML.MLUpdateProgressEventTrainingBegin, 1 << 0)
        self.assertEqual(CoreML.MLUpdateProgressEventEpochEnd, 1 << 1)
        self.assertEqual(CoreML.MLUpdateProgressEventMiniBatchEnd, 1 << 2)



import objc

from PyObjCTools.TestSupport import TestCase, min_sdk_level
import CoreML


class TestMLCustomLayerHelper(CoreML.NSObject):
    def initWithParameterDictionary_error_(self, p, e):
        pass

    def setWeightData_error_(self, w, e):
        return 1

    def outputShapesForInputShapes_error_(self, w, e):
        return 1

    def evaluateOnCPUWithInputs_outputs_error_(self, i, o, e):
        return 1

    def encodeToCommandBuffer_inputs_outputs_error_(self, b, i, o, e):
        return 1


class TestMLCustomLayer(TestCase):
    @min_sdk_level("10.13")
    def testProtocols(self):
        objc.protocolNamed("MLCustomLayer")

    def testMethods(self):
        self.assertArgHasType(
            TestMLCustomLayerHelper.initWithParameterDictionary_error_, 1, b"o^@"
        )

        self.assertArgHasType(TestMLCustomLayerHelper.setWeightData_error_, 1, b"o^@")
        self.assertResultIsBOOL(TestMLCustomLayerHelper.setWeightData_error_)

        self.assertArgHasType(
            TestMLCustomLayerHelper.outputShapesForInputShapes_error_, 1, b"o^@"
        )

        self.assertArgHasType(
            TestMLCustomLayerHelper.evaluateOnCPUWithInputs_outputs_error_, 2, b"o^@"
        )
        self.assertResultIsBOOL(
            TestMLCustomLayerHelper.evaluateOnCPUWithInputs_outputs_error_
        )

        self.assertArgHasType(
            TestMLCustomLayerHelper.encodeToCommandBuffer_inputs_outputs_error_,
            3,
            b"o^@",
        )
        self.assertResultIsBOOL(
            TestMLCustomLayerHelper.encodeToCommandBuffer_inputs_outputs_error_
        )



"""
Python mapping for the CoreML framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""

import sys

import Foundation
import objc
from CoreML import _metadata
from CoreML import _CoreML

sys.modules["CoreML"] = mod = objc.ObjCLazyModule(
    "CoreML",
    "com.apple.CoreML",
    objc.pathForFramework("/System/Library/Frameworks/CoreML.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "objc": objc,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
    },
    (_CoreML, Foundation),
)


del sys.modules["CoreML._metadata"]



from PyObjCTools.TestSupport import TestCase
import CoreML


class TestMLFeatureType(TestCase):
    def test_enum_types(self):
        self.assertIsEnumType(CoreML.MLFeatureType)

    def testConstants(self):
        self.assertEqual(CoreML.MLFeatureTypeInvalid, 0)
        self.assertEqual(CoreML.MLFeatureTypeInt64, 1)
        self.assertEqual(CoreML.MLFeatureTypeDouble, 2)
        self.assertEqual(CoreML.MLFeatureTypeString, 3)
        self.assertEqual(CoreML.MLFeatureTypeImage, 4)
        self.assertEqual(CoreML.MLFeatureTypeMultiArray, 5)
        self.assertEqual(CoreML.MLFeatureTypeDictionary, 6)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLPredictionOptions(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertResultIsBOOL(CoreML.MLPredictionOptions.usesCPUOnly)
        self.assertArgIsBOOL(CoreML.MLPredictionOptions.setUsesCPUOnly_, 0)



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLUpdateTask(TestCase):
    @min_os_level("10.15")
    def test_methods10_15(self):
        self.assertArgIsBlock(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_configuration_completionHandler_error_,
            3,
            b"v@",
        )
        self.assertArgIsOut(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_configuration_completionHandler_error_,
            4,
        )

        self.assertArgIsOut(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_configuration_progressHandlers_error_,
            4,
        )

    @min_os_level("11.0")
    def test_methods11_0(self):
        self.assertArgIsBlock(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_completionHandler_error_,
            2,
            b"v@",
        )
        self.assertArgIsOut(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_completionHandler_error_,
            3,
        )

        self.assertArgIsOut(
            CoreML.MLUpdateTask.updateTaskForModelAtURL_trainingData_progressHandlers_error_,
            3,
        )



import objc

from PyObjCTools.TestSupport import TestCase, min_sdk_level
import CoreML


class TestMLBatchProviderHelper(CoreML.NSObject):
    def count(self):
        return 1

    def featuresAtIndex_(self, i):
        return "a"


class TestMLBatchProvider(TestCase):
    @min_sdk_level("10.14")
    def testProtocols(self):
        objc.protocolNamed("MLBatchProvider")
        objc.protocolNamed("MLFeatureProvider")

    def testMethods(self):
        self.assertResultHasType(TestMLBatchProviderHelper.count, objc._C_NSInteger)
        self.assertArgHasType(
            TestMLBatchProviderHelper.featuresAtIndex_, 0, objc._C_NSInteger
        )



from PyObjCTools.TestSupport import TestCase, min_os_level
import CoreML


class TestMLFeatureDescription(TestCase):
    def test_typed_enum(self):
        self.assertIsTypedEnum(CoreML.MLModelMetadataKey, str)

    @min_os_level("10.13")
    def testConstants(self):
        self.assertIsInstance(CoreML.MLModelDescriptionKey, str)
        self.assertIsInstance(CoreML.MLModelVersionStringKey, str)
        self.assertIsInstance(CoreML.MLModelAuthorKey, str)
        self.assertIsInstance(CoreML.MLModelLicenseKey, str)
        self.assertIsInstance(CoreML.MLModelCreatorDefinedKey, str)
